"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = build;

var _package = require("../package");

var _project = require("../project");

var _path = _interopRequireDefault(require("path"));

var _rollup = require("./rollup");

var _aliases = require("./aliases");

var logger = _interopRequireWildcard(require("../logger"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _messages = require("../messages");

var _errors = require("../errors");

var _config = require("./config");

var _workerClient = require("../worker-client");

var _babel = require("../rollup-plugins/babel");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let browserPattern = /typeof\s+(window|document)/;

async function buildPackage(pkg, aliases) {
  let configs = (0, _config.getRollupConfigs)(pkg, aliases);
  await Promise.all([fs.remove(_path.default.join(pkg.directory, "dist")), ...pkg.entrypoints.map(entrypoint => {
    return fs.remove(_path.default.join(entrypoint.directory, "dist"));
  })]); // TODO: Fix all this stuff to work with multiple entrypoints

  let hasCheckedBrowser = pkg.entrypoints[0].browser !== null;
  await Promise.all(configs.map(async ({
    config,
    outputs
  }) => {
    // $FlowFixMe this is not a problem with flow, i did something wrong but it's not worth fixing right now
    let bundle = await (0, _rollup.rollup)(config);
    let result = await Promise.all(outputs.map(outputConfig => {
      return bundle.write(outputConfig);
    }));
    const nodeDevOutput = result[0].output;

    if (!hasCheckedBrowser) {
      let allCode = nodeDevOutput.map(({
        code
      }) => code).join("\n");
      hasCheckedBrowser = true;

      if (browserPattern.test(allCode)) {
        throw (async () => {
          let shouldAddBrowserField = await _messages.confirms.addBrowserField(pkg);

          if (shouldAddBrowserField) {
            pkg.setFieldOnEntrypoints("browser");
            await Promise.all(pkg.entrypoints.map(x => x.save()));
          } else {
            throw new _errors.FatalError(_messages.errors.deniedWriteBrowserField, pkg);
          }
        })();
      }
    }
  }));
}

async function retryableBuild(pkg, aliases) {
  try {
    await buildPackage(pkg, aliases);
  } catch (err) {
    if (err instanceof Promise) {
      await err;
      await retryableBuild(pkg, aliases);
      return;
    }

    throw err;
  }
}

async function build(directory) {
  // do more stuff with checking whether the repo is using yarn workspaces or bolt
  try {
    (0, _workerClient.createWorker)();
    await _babel.hasherPromise;
    let project = await _project.Project.create(directory);
    logger.info("building bundles!");
    let aliases = (0, _aliases.getAliases)(project);
    await Promise.all(project.packages.map(pkg => retryableBuild(pkg, aliases)));
    logger.success("built bundles!");
  } finally {
    (0, _workerClient.destroyWorker)();
  }
}