"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = dev;

var _project = require("./project");

var _logger = require("./logger");

var fs = _interopRequireWildcard(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

let tsExtensionPattern = /tsx?$/;

async function getTypeSystem(entrypoint) {
  if (tsExtensionPattern.test(entrypoint.source)) {
    return "typescript";
  } // TODO: maybe we should write the flow symlink even if there isn't an 
  // comment so that if someone adds an  comment they don't have to run preconstruct dev again


  let content = await fs.readFile(entrypoint.source, "utf8");

  if (content.includes("@flow")) {
    return "flow";
  }

  return null;
}

async function writeFlowFile(typeSystemPromise, entrypoint) {
  let typeSystem = await typeSystemPromise;

  if (typeSystem !== null) {
    let cjsDistPath = _path.default.join(entrypoint.directory, entrypoint.main);

    await fs.symlink(entrypoint.source, cjsDistPath + (typeSystem === "flow" ? ".flow" : ".ts"));
  }
}

async function dev(projectDir) {
  let project = await _project.Project.create(projectDir);
  project.packages.forEach(({
    entrypoints
  }) => entrypoints.forEach(x => x.strict()));
  (0, _logger.info)("project is valid!");
  let promises = [];
  await Promise.all(project.packages.map(pkg => {
    return Promise.all(pkg.entrypoints.map(async _entrypoint => {
      let entrypoint = _entrypoint.strict();

      let typeSystemPromise = getTypeSystem(entrypoint);
      await fs.remove(_path.default.join(entrypoint.directory, "dist"));
      await fs.ensureDir(_path.default.join(entrypoint.directory, "dist"));
      let promises = [writeFlowFile(typeSystemPromise, entrypoint), fs.writeFile(_path.default.join(entrypoint.directory, entrypoint.main), `'use strict';

let unregister = require('${require.resolve("@preconstruct/hook")}').___internalHook('${project.directory}');

module.exports = require('${entrypoint.source}');

unregister();
`)];

      if (entrypoint.module) {
        promises.push(fs.symlink(entrypoint.source, _path.default.join(entrypoint.directory, entrypoint.module)));
      }

      let browserField = entrypoint.browser;

      if (browserField) {
        for (let key of Object.keys(browserField)) {
          promises.push(fs.symlink(entrypoint.source, _path.default.join(entrypoint.directory, browserField[key])));
        }
      }

      let rnField = entrypoint.reactNative;

      if (rnField) {
        for (let key of Object.keys(rnField)) {
          promises.push(fs.symlink(entrypoint.source, _path.default.join(entrypoint.directory, rnField[key])));
        }
      }

      return Promise.all(promises);
    }));
  }));
  await Promise.all(promises);
  (0, _logger.success)("created links!");
}