"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StrictEntrypoint = exports.Entrypoint = void 0;

var _sarcastic = _interopRequireDefault(require("sarcastic"));

var _path = _interopRequireDefault(require("path"));

var _validate = require("./validate");

var _item = require("./item");

var _validatePackage = require("./validate-package");

var _resolve = _interopRequireDefault(require("resolve"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

let fields = ["version", "description", "main", "module", "umd:main", "browser", "react-native"];

function setFieldInOrder(obj, field, value) {
  if (field in obj) {
    return _objectSpread({}, obj, {
      [field]: value
    });
  }

  let fieldIndex = fields.indexOf(field);
  let idealField = fields.slice(0, fieldIndex).reverse().find(key => {
    return key in obj;
  });

  if (idealField === undefined) {
    return _objectSpread({}, obj, {
      [field]: value
    });
  }

  let newObj = {};

  for (let key in obj) {
    newObj[key] = obj[key];

    if (key === idealField) {
      newObj[field] = value;
    }
  }

  return newObj;
}
/*::
import { Package } from './package'
*/


class Entrypoint extends _item.Item {
  constructor(filePath, contents, pkg) {
    super(filePath, contents);

    _defineProperty(this, "package", void 0);

    _defineProperty(this, "_strict", void 0);

    this.package = pkg;
  }

  get name() {
    return _path.default.join(this.package.name, _path.default.relative(this.package.directory, this.directory));
  }

  get main() {
    return (0, _sarcastic.default)(this.json.main, _sarcastic.default.maybe(_sarcastic.default.string));
  }

  set main(path) {
    this.json = setFieldInOrder(this.json, "main", path);
  }

  get module() {
    return (0, _sarcastic.default)(this.json.module, _sarcastic.default.maybe(_sarcastic.default.string));
  }

  set module(path) {
    this.json = setFieldInOrder(this.json, "module", path);
  }

  get browser() {
    return (0, _sarcastic.default)(this.json.browser, _sarcastic.default.maybe(_sarcastic.default.either(_sarcastic.default.string, _sarcastic.default.objectOf(_sarcastic.default.string))));
  }

  set browser(option) {
    this.json = setFieldInOrder(this.json, "browser", option);
  }

  get reactNative() {
    return (0, _sarcastic.default)(this.json["react-native"], _sarcastic.default.maybe(_sarcastic.default.either(_sarcastic.default.string, _sarcastic.default.objectOf(_sarcastic.default.string))));
  }

  set reactNative(option) {
    this.json = setFieldInOrder(this.json, "react-native", option);
  }

  get umdMain() {
    return (0, _sarcastic.default)(this.json["umd:main"], _sarcastic.default.maybe(_sarcastic.default.string));
  }

  set umdMain(path) {
    this.json = setFieldInOrder(this.json, "umd:main", path);
  }

  get configSource() {
    return (0, _sarcastic.default)(this._config.source, _sarcastic.default.default(_sarcastic.default.string, "src/index"));
  }

  get source() {
    return _resolve.default.sync(_path.default.join(this.directory, this.configSource), {
      extensions: _constants.EXTENSIONS
    });
  }

  get umdName() {
    return (0, _sarcastic.default)(this._config.umdName, _sarcastic.default.maybe(_sarcastic.default.string));
  }

  set umdName(umdName) {
    if (umdName === null) {
      delete this._config.umdName;
    } else {
      this._config.umdName = umdName;
    }
  }

  strict() {
    if (!this._strict) {
      (0, _validatePackage.validatePackage)(this.package);
      (0, _validate.validateEntrypoint)(this, false);
      this._strict = new StrictEntrypoint(this.path, this._contents, this.package);
    }

    return this._strict;
  }

}

exports.Entrypoint = Entrypoint;

class StrictEntrypoint extends Entrypoint {
  get main() {
    return (0, _sarcastic.default)(this.json.main, _sarcastic.default.string);
  }

  set main(path) {
    this.json = setFieldInOrder(this.json, "main", path);
  }

  get browser() {
    return (0, _sarcastic.default)(this.json.browser, _sarcastic.default.maybe(_sarcastic.default.objectOf(_sarcastic.default.string)));
  }

  set browser(option) {
    this.json = setFieldInOrder(this.json, "browser", option);
  }

  get reactNative() {
    return (0, _sarcastic.default)(this.json["react-native"], _sarcastic.default.maybe(_sarcastic.default.objectOf(_sarcastic.default.string)));
  }

  set reactNative(option) {
    this.json = setFieldInOrder(this.json, "react-native", option);
  }

  updater(json) {
    super.updater(json);
    (0, _validatePackage.validatePackage)(this.package);
    (0, _validate.validateEntrypoint)(this, false);
  }

  strict() {
    return this;
  }

}

exports.StrictEntrypoint = StrictEntrypoint;