"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = fix;

var _entrypoint = require("./entrypoint");

var _project = require("./project");

var _prompt = require("./prompt");

var _logger = require("./logger");

var _messages = require("./messages");

var _validate = require("./validate");

var _validatePackage = require("./validate-package");

async function fixEntrypoint(entrypoint) {
  (0, _validate.validateEntrypointSource)(entrypoint);

  if (entrypoint.umdMain !== null && !(0, _validate.isUmdNameSpecified)(entrypoint)) {
    let umdName = await (0, _prompt.promptInput)(_messages.inputs.getUmdName, entrypoint);
    entrypoint.umdName = umdName;
    await entrypoint.save();
    return true;
  }

  return false;
}

async function fix(directory) {
  let {
    packages
  } = await _project.Project.create(directory);
  let didModify = (await Promise.all(packages.map(async pkg => {
    let didModifyInPkgFix = await (0, _validatePackage.fixPackage)(pkg);
    let didModifyInEntrypointsFix = (await Promise.all(pkg.entrypoints.map(fixEntrypoint))).some(x => x);
    return didModifyInPkgFix || didModifyInEntrypointsFix;
  }))).some(x => x);
  (0, _logger.success)(didModify ? "fixed project!" : "project already valid!");
}