"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Item = void 0;

var fs = _interopRequireWildcard(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _sarcastic = _interopRequireDefault(require("sarcastic"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

let itemsByPath = {};

class Item {
  constructor(filePath, contents) {
    _defineProperty(this, "_contents", void 0);

    _defineProperty(this, "_stringifiedSavedJson", void 0);

    _defineProperty(this, "path", void 0);

    _defineProperty(this, "directory", void 0);

    _defineProperty(this, "json", void 0);

    _defineProperty(this, "_config", void 0);

    this.json = (0, _sarcastic.default)(JSON.parse(contents), _sarcastic.default.object);
    this._stringifiedSavedJson = JSON.stringify(this.json, null, 2);
    this._contents = contents;
    this.path = filePath;
    this.directory = _path.default.dirname(filePath);
    this._config = this.json[_constants.PKG_JSON_CONFIG_FIELD] || {};

    if (itemsByPath[this.path] === undefined) {
      itemsByPath[this.path] = new Set();
    }

    itemsByPath[this.path].add(this);
  }

  updater(json) {
    this.json = json;
  }

  async refresh() {
    let contents = await fs.readFile(this.path, "utf-8");
    let json = (0, _sarcastic.default)(JSON.parse(contents), _sarcastic.default.object);

    for (let item of itemsByPath[this.path]) {
      item.updater(json);
    }
  }

  async save() {
    if (Object.keys(this._config).length) {
      this.json[_constants.PKG_JSON_CONFIG_FIELD] = this._config;
    } else {
      delete this.json[_constants.PKG_JSON_CONFIG_FIELD];
    }

    let stringified = JSON.stringify(this.json, null, 2);

    if (stringified !== this._stringifiedSavedJson) {
      await fs.writeFile(this.path, JSON.stringify(this.json, null, 2) + "\n");
      this._config = this.json[_constants.PKG_JSON_CONFIG_FIELD] || {};

      for (let item of itemsByPath[this.path]) {
        item.updater(this.json);
      }

      this._stringifiedSavedJson = stringified;
      return true;
    }

    return false;
  }

}

exports.Item = Item;