"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.hasherPromise = void 0;

var babel = _interopRequireWildcard(require("@babel/core"));

var _rollupPluginutils = require("rollup-pluginutils");

var _workerClient = require("../../worker-client");

var _xxhashWasm = _interopRequireDefault(require("xxhash-wasm"));

var _quickLru = _interopRequireDefault(require("quick-lru"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const regExpCharactersRegExp = /[\\^$.*+?()[\]{}|]/g;

const escapeRegExpCharacters = str => str.replace(regExpCharactersRegExp, "\\$&");

const unpackOptions = (_ref = {}) => {
  let {
    extensions = babel.DEFAULT_EXTENSIONS,
    // rollup uses sourcemap, babel uses sourceMaps
    // just normalize them here so people don't have to worry about it
    sourcemap = true,
    sourcemaps = true,
    sourceMap = true,
    sourceMaps = true
  } = _ref,
      rest = _objectWithoutProperties(_ref, ["extensions", "sourcemap", "sourcemaps", "sourceMap", "sourceMaps"]);

  return _objectSpread({
    extensions,
    plugins: [],
    sourceMaps: sourcemap && sourcemaps && sourceMap && sourceMaps
  }, rest, {
    caller: _objectSpread({
      name: "rollup-plugin-babel",
      supportsStaticESM: true,
      supportsDynamicImport: true
    }, rest.caller)
  });
};

const lru = new _quickLru.default({
  maxSize: 1000
});
let hasher;
let hasherPromise = (0, _xxhashWasm.default)().then(({
  h64
}) => {
  hasher = h64;
});
exports.hasherPromise = hasherPromise;

let rollupPluginBabel = pluginOptions => {
  const _unpackOptions = unpackOptions(pluginOptions),
        {
    exclude,
    extensions,
    include
  } = _unpackOptions,
        babelOptions = _objectWithoutProperties(_unpackOptions, ["exclude", "extensions", "include"]);

  const extensionRegExp = new RegExp(`(${extensions.map(escapeRegExpCharacters).join("|")})$`);
  const includeExcludeFilter = (0, _rollupPluginutils.createFilter)(include, exclude);

  const filter = id => extensionRegExp.test(id) && includeExcludeFilter(id);

  return {
    name: "babel",

    transform(code, filename) {
      if (!filter(filename)) return Promise.resolve(null);
      let hash = hasher(filename);

      if (lru.has(hash)) {
        let cachedResult = lru.get(hash);

        if (code === cachedResult.code) {
          return cachedResult.promise;
        }
      }

      let options = JSON.stringify(_objectSpread({}, babelOptions, {
        filename
      }));
      let promise = (0, _workerClient.getWorker)().transformBabel(code, options);
      lru.set(hash, {
        code,
        promise
      });
      return promise;
    }

  };
};

var _default = rollupPluginBabel;
exports.default = _default;