"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = flowAndNodeDevProdEntry;

var _path = _interopRequireDefault(require("path"));

var _utils = require("../build/utils");

var _utils2 = require("../utils");

var _package = require("../package");

var _errors = require("../errors");

var fs = _interopRequireWildcard(require("fs-extra"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function flowAndNodeDevProdEntry(pkg) {
  return {
    name: "flow-and-prod-dev-entry",

    async resolveId(source, importer) {
      let resolved = await this.resolve(source, importer, {
        skipSelf: true
      });

      if (resolved.id.startsWith("\0") || resolved.id.startsWith(pkg.directory)) {
        return resolved;
      }

      throw new _errors.FatalError(`all relative imports in a package should only import modules inside of their package directory but "${_path.default.relative(pkg.directory, importer)}" is importing "${source}"`, pkg);
    },

    // eslint-disable-next-line no-unused-vars
    async generateBundle(opts, bundle, something) {
      for (const n in bundle) {
        const file = bundle[n]; // $FlowFixMe

        let facadeModuleId = file.facadeModuleId;

        if (file.isAsset || !file.isEntry || facadeModuleId == null) {
          continue;
        }

        let mainFieldPath = file.fileName.replace(/\.prod\.js$/, ".js");

        let relativeToSource = _path.default.relative(_path.default.dirname(_path.default.join(opts.dir, file.fileName)), facadeModuleId);

        let isEntrySourceTypeScript = /\.tsx?$/.test(facadeModuleId);

        if (!isEntrySourceTypeScript) {
          let flowMode = false;
          let source = await fs.readFile(facadeModuleId, "utf8");

          if (source.includes("@flow")) {
            flowMode = file.exports.includes("default") ? "all" : "named";
          }

          if (flowMode !== false) {
            let flowFileSource = (0, _utils2.flowTemplate)(flowMode === "all", relativeToSource);
            let flowFileName = mainFieldPath + ".flow";
            bundle[flowFileName] = {
              fileName: flowFileName,
              isAsset: true,
              source: flowFileSource
            };
          }
        }

        let mainEntrySource = `'use strict';

if (process.env.NODE_ENV === "production") {
  module.exports = require("./${_path.default.basename((0, _utils.getProdPath)(mainFieldPath))}");
} else {
  module.exports = require("./${_path.default.basename((0, _utils.getDevPath)(mainFieldPath))}");
}\n`;
        bundle[mainFieldPath] = {
          fileName: mainFieldPath,
          isAsset: true,
          source: mainEntrySource
        };
      }
    }

  };
}