"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDeclarationCreator = createDeclarationCreator;

var _resolveFrom = _interopRequireDefault(require("resolve-from"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _weakMemoize = _interopRequireDefault(require("@emotion/weak-memoize"));

var _memoize = _interopRequireDefault(require("@emotion/memoize"));

var _path = _interopRequireDefault(require("path"));

var _languageServiceHost = require("./language-service-host");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let unsafeRequire = require;
let getService = (0, _weakMemoize.default)(typescript => (0, _memoize.default)(async configFileName => {
  console.log("creating service");
  let configFileContents = await fs.readFile(configFileName, "utf8");
  const result = typescript.parseConfigFileTextToJson(configFileName, configFileContents);
  let thing = typescript.parseJsonConfigFileContent(result, typescript.sys, process.cwd(), undefined, configFileName);
  let LanguageServiceHostClass = (0, _languageServiceHost.createLanguageServiceHostClass)(typescript);
  let servicesHost = new LanguageServiceHostClass(thing, []);
  let service = typescript.createLanguageService(servicesHost, typescript.createDocumentRegistry());
  servicesHost.setLanguageService(service);
  return service;
}));

async function createDeclarationCreator(dirname) {
  let typescript;

  try {
    typescript = unsafeRequire((0, _resolveFrom.default)(dirname, "typescript"));
  } catch (err) {
    if (err.code === "MODULE_NOT_FOUND") {
      throw new Error("an entrypoint source file ends with the .ts or .tsx extension but the typescript module could not be resolved from the project directory, please install it.");
    }

    throw err;
  }

  let configFileName = typescript.findConfigFile(dirname, typescript.sys.fileExists);

  if (!configFileName) {
    throw new Error("an entrypoint source file ends with the .ts extension but no TypeScript config exists, please create one.");
  }

  let service = await getService(typescript)(configFileName);
  return filename => {
    let output = service.getEmitOutput(filename, true);
    return {
      name: output.outputFiles[0].name.replace(dirname, _path.default.join(dirname, "dist", "declarations")),
      content: output.outputFiles[0].text
    };
  };
}