"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = typescriptDeclarations;

var _path = _interopRequireDefault(require("path"));

var _errors = require("../../errors");

var _package = require("../../package");

var _createGenerator = require("./create-generator");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let isTsPath = source => /\.tsx?/.test(source);

function tsTemplate(hasDefaultExport, relativePath) {
  return `export * from "${relativePath}";${hasDefaultExport ? `\nexport { default } from "${relativePath}";` : ""}\n`;
}

function typescriptDeclarations(pkg) {
  if (!pkg.entrypoints.some(({
    source
  }) => isTsPath(source))) {
    return {
      name: "typescript-declarations"
    };
  }

  return {
    name: "typescript-declarations",

    // eslint-disable-next-line no-unused-vars
    async generateBundle(opts, bundle, something) {
      let creator = await (0, _createGenerator.createDeclarationCreator)(pkg.directory);
      let srcFilenameToDtsFilenameMap = new Map();
      let doneModules = new Set();

      for (const n in bundle) {
        const file = bundle[n];

        if (file.isAsset) {
          continue;
        }

        for (let mod in file.modules) {
          if (!doneModules.has(mod)) {
            doneModules.add(mod);
            let {
              name,
              content
            } = creator(mod);
            srcFilenameToDtsFilenameMap.set(mod, name);
            bundle[name] = {
              fileName: name,
              isAsset: true,
              source: content
            };
          }
        }
      }

      for (const n in bundle) {
        const file = bundle[n]; // $FlowFixMe

        let facadeModuleId = file.facadeModuleId;

        if (file.isAsset || !file.isEntry || facadeModuleId == null) {
          continue;
        }

        let dtsFilename = srcFilenameToDtsFilenameMap.get(facadeModuleId);

        if (!dtsFilename) {
          throw new _errors.FatalError(`no .d.ts file was found for the entrypoint at ${facadeModuleId}`, pkg);
        }

        let mainFieldPath = file.fileName.replace(/\.prod\.js$/, "");

        let relativeToSource = _path.default.relative(_path.default.dirname(_path.default.join(opts.dir, file.fileName)), dtsFilename.replace(/\.d\.ts$/, ""));

        if (!relativeToSource.startsWith(".")) {
          relativeToSource = `./${relativeToSource}`;
        }

        let tsFileSource = tsTemplate(file.exports.includes("default"), relativeToSource);
        let tsFileName = mainFieldPath + ".d.ts";
        bundle[tsFileName] = {
          fileName: tsFileName,
          isAsset: true,
          source: tsFileSource
        };
      }
    }

  };
}