"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getNameForDist = getNameForDist;
exports.getValidStringFieldContentForBuildType = getValidStringFieldContentForBuildType;
exports.getValidObjectFieldContentForBuildType = getValidObjectFieldContentForBuildType;
exports.flowTemplate = flowTemplate;

function getNameForDist(name) {
  return name.replace(/.*\//, "");
}

function getValidStringFieldContentForBuildType(type, pkgName) {
  let safeName = getNameForDist(pkgName);

  switch (type) {
    case "main":
      {
        return `dist/${safeName}.cjs.js`;
      }

    case "module":
      {
        return `dist/${safeName}.esm.js`;
      }

    case "umd:main":
      {
        return `dist/${safeName}.umd.min.js`;
      }
  }

  throw new Error(`unknown string build type: ${type}. this is likely a bug in preconstruct.`);
}

function getValidObjectFieldContentForBuildType(type, pkgName, hasModuleBuild) {
  let safeName = getNameForDist(pkgName);

  switch (type) {
    case "browser":
      {
        let obj = {
          [`./dist/${safeName}.cjs.js`]: `./dist/${safeName}.browser.cjs.js`
        };

        if (hasModuleBuild) {
          obj[`./dist/${safeName}.esm.js`] = `./dist/${safeName}.browser.esm.js`;
        }

        return obj;
      }

    case "react-native":
      {
        let obj = {
          [`./dist/${safeName}.cjs.js`]: `./dist/${safeName}.native.cjs.js`
        };

        if (hasModuleBuild) {
          obj[`./dist/${safeName}.esm.js`] = `./dist/${safeName}.native.esm.js`;
        }

        return obj;
      }
  }

  throw new Error(`unknown object build type: ${type}. this is likely a bug in preconstruct.`);
}

function flowTemplate(hasDefaultExport, relativePath) {
  return `// @flow
export * from "${relativePath}";${hasDefaultExport ? `\nexport { default } from "${relativePath}";` : ""}\n`;
}